//
// Name: MFACE.PRG
// Project: Fischer Techniek interfacing
// Description: ft multiface interface driver
// Language: Clipper 5.2
// Author: Rene Nooteboom
// Date created: 21-08-2005
// Date updated: 29-08-2005
//
//  default values :
//
//  parallel port  I/O   IRQ
//
//   LPT1         378H    7
//   LPT2         278H    5
//   LPT3         3BCH    -
//   LPT4         2BCH    -
//
//  example mface.ini file :  //
//                            LPT3_IO = h378 : hex
//                            LPT3_IO = A99 : assumes hex
//                            LPT2_IO = 632 : assumes decimal
//                            //
//
#include "generic.ch"
#include "noot.ch"
#include "ft.ch"
//
STATIC cHexLptIO_:={"378","278","3BC","2BC"} // max 4 lptports
STATIC nLptStatusReg_ := {889,633,957,701} // offset 1 from dataregister
STATIC nLptControlReg_ := {890,634,958,702} // offset 2 from dataregister
STATIC lLptUsed_ := {FALSE,FALSE,FALSE,FALSE},nMotorByte_:={0,0,0,0}
//
FUNCTION MfInit(nLptPort,xIOAddress,laskvalues)
//
// all arguments optional,xIOAddress can be numeric decimal or a hex-string
//
 LOCAL cmsg,getlist:={},nStatusRegIO,nControlRegIO
 DEFAULT nLptPort TO 1,laskvalues TO TRUE
 IF xIOAddress == NIL
  xIOAddress := MfGetIOAddress(nLptPort)
 ENDIF
 IF IS_NUM(xIOAddress) // convert to hex
  xIOAddress := padl(alltrim(ntoc(xIOAddress,16,3)),3,"0")
 ENDIF
 IF laskvalues
  save_env({9,5,12,40})
  color_set(CR_GETS)
  @ 9,5 CLEAR TO 12,40
  @ 9,5 TO 12,40
  @ 10,6 SAY "LPT-port(1 to 4) :"
  @ 11,6 SAY "IO-Address(hex)  :"
  @ 10,30 GET nLptPort PICTURE "9" VALID IS_BETW(nLptPort,1,4)
  @ 11,30 GETX xIOAddress MESSAGE "hexadecimal" PICTURE "@! XXX" ;
   VALID IS_HEX(xIOAddress) ERROR "not hex"
  READ
  rest_env()
  IF lastkey() == K_ESC
   nLptPort := -1
  ENDIF
 ENDIF
 IF nLptPort > 0
  nStatusRegIO := cton(xIOAddress,16) + 1
  nControlRegIO := nStatusRegIO + 1
  IF outbyte(nControlRegIO,"8E") // initialize interface : 142d
   cHexLptIO_[nLptPort] := xIOAddress
   nLptStatusReg_[nLptPort] := nStatusRegIO
   nLptControlReg_[nLptPort] := nControlRegIO
   lLptUsed_[nLptPort] := TRUE
   MfOutput(nLptPort,0) // switch all motors off
   cmsg := "MfInit(): initialized LPT" + str(nLptPort,1,0) + ", IO: " + ;
    xIOAddress + "h"
   rowmessage(cmsg,,,CR_MESSAGE,2)
   MfShowUsedIF()
  ELSE
   cmsg := "MfInit(): failed to initialize LPT" + str(nLptPort,1,0) + ;
    ",IO: " + xIOAddress + "h"
   rowmessage(cmsg,,,CR_WARNING,4,TRUE)
   nLptPort := -1
  ENDIF
 ELSE
  nLptPort := -1
 ENDIF
RETURN nLptPort
//
PROCEDURE MfClose(nLptPort)
 DEFAULT nLptPort TO 1
 IF nLptPort > 0 .AND. lLptUsed_[nLptPort]
  MfOutput(nLptPort,0)
  outbyte(nLptControlReg_[nLptPort],"8C") // 140d=1000 1100
  lLptUsed_[nLptPort] := FALSE
 ENDIF
RETURN
//
FUNCTION MfInput(nLptPort)
 LOCAL nbyte1,nbyte2,ntmp1,ntmp2,nbyte,cmsg
 DEFAULT nLptPort TO 1
 IF outbyte(nLptControlReg_[nLptPort],"8E") // 142d=1000 1110
  nbyte1 := inbyte(nLptStatusReg_[nLptPort])
  IF !(nbyte1 == -1)
   nbyte1 := numand(nbyte1,"E8") // 1110 1000
   nbyte1 := numxor(nbyte1,"80") // 1000 0000
   ntmp1 := numand(nbyte1,"E0") // 1110 0000
   ntmp2 := numand(nbyte1,"08") * 2  // 0000 1000 and SHL 1
   nbyte1 := numor(ntmp1,ntmp2)
   IF outbyte(nLptControlReg_[nLptPort],"8F") // 143d=1000 1111
    nbyte2 := inbyte(nLptStatusReg_[nLptPort])
    IF !(nbyte2 == -1)
     nbyte2 := numand(nbyte2,"E8") // 1110 1000
     nbyte2 := numxor(nbyte2,"80") // 1000 0000
     ntmp1 := numand(nbyte2,"E0") // 1110 0000
     ntmp2 := numand(nbyte2,"08") * 2  // 0000 1000 and SHL 1
     nbyte2 := numor(ntmp1,ntmp2) / 16 // SHR 4
     nbyte := numor(nbyte1,nbyte2)
    ELSE
     nbyte := -1
     cmsg := "MfInput(): Error reading port: " + ;
      ntoc(nLptStatusReg_[nLptport],16,3) + "h"
     rowmessage(cmsg,,,CR_WARNING,4,TRUE)
    ENDIF
   ELSE
    nbyte := -1
    cmsg := "MfInput(): Error writing port: " + ;
     ntoc(nLptControlReg_[nLptport],16,3) + "h"
    rowmessage(cmsg,,,CR_WARNING,4,TRUE)
   ENDIF
  ELSE
   nbyte := -1
   cmsg := "MfInput(): Error reading port: " + ;
    ntoc(nLptStatusReg_[nLptport],16,3) + "h"
   rowmessage(cmsg,,,CR_WARNING,4,TRUE)
  ENDIF
 ELSE
  nbyte := -1
  cmsg := "MfInput(): Error writing port: " + ;
   ntoc(nLptControlReg_[nLptport],16,3) + "h"
  rowmessage(cmsg,,,CR_WARNING,4,TRUE)
 ENDIF
 outbyte(nLptControlReg_[nLptPort],"8C") // 140d=1000 1100
RETURN nbyte
//
PROCEDURE MfOutput(nLptPort,nByte)
 LOCAL cmsg
 DEFAULT nLptport TO 1
 IF outbyte(nLptControlReg_[nLptPort],"8E") // 142d=1000 1110 initialize
  IF outbyte(cHexLptIO_[nLptPort],nByte)
   outbyte(nLptControlReg_[nLptPort],"8C") // 140d=1000 1100
   nMotorByte_[nLptport] := nByte
  ELSE
   cmsg := "MfOutput(): Error writing to port: " + ;
    cHexLptIO_[nLptPort] + "h"
   rowmessage(cmsg,,,CR_WARNING,4,TRUE)
  ENDIF
 ELSE
  cmsg := "MfOutput(): Error writing to port: " + ;
   ntoc(nLptControlReg_[nLptPort],16,3) + "h"
  rowmessage(cmsg,,,CR_WARNING,4,TRUE)
 ENDIF
RETURN
//
FUNCTION MfSensor(nLptPort,nE1toE8)
 LOCAL nInput:=0,lSensor:=FALSE
 nInput := MfInput(nLptPort)
 lSensor := isbit(nInput,nE1toE8)
RETURN lSensor
//
PROCEDURE MfMotorsOff(nLptPort)
 MfOutput(nLptPort,0)
RETURN
//
PROCEDURE MfMotor(nLptPort,nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
 LOCAL nOutputByte
 DEFAULT nLptPort TO 1
 nOutputByte := nMotorByte_[nLptPort]
 DO CASE
  CASE nMotorNr == 1
   nOutputByte := numand(nOutputByte,"FC") // 1111 1100
   nOutputByte := numor(nOutputByte,nDirection) // no shift
  CASE nMotorNr == 2
   nOutputByte := numand(nOutputByte,"F3") // 1111 0011
   nOutputByte := numor(nOutputByte,nDirection*4) // left shift 2 bits
  CASE nMotorNr == 3
   nOutputByte := numand(nOutputByte,"CF") // 1100 1111
   nOutputByte := numor(nOutputByte,nDirection*16) // left shift 4 bits
  CASE nMotorNr == 4
   nOutputByte := numand(nOutputByte,"3F") // 0011 1111
   nOutputByte := numor(nOutputByte,nDirection*64) // left shift 6 bits
  OTHERWISE
 ENDCASE
 MfOutput(nLptPort,nOutputByte)
RETURN
//
PROCEDURE MfButton(nLptPort,nE1toE8)
 DO WHILE !MfSensor(nLptPort,nE1toE8)
 ENDDO
RETURN
//
PROCEDURE MfButtonPushed(nLptPort,nE1toE8)
 DO WHILE MfSensor(nLptPort,nE1toE8)
 ENDDO
RETURN
//
PROCEDURE MfOnNr(nLptPort,nO1toO8) // Ausgang # sturen
 LOCAL nByte := nMotorByte_[nLptPort]
 DEFAULT nLptPort TO 1
 nByte := setbit(nByte,nO1toO8)
 MfOutput(nLptPort,nByte)
RETURN
//
PROCEDURE MfOffNr(nLptPort,nO1toO8)  // Ausgang # sturen
 LOCAL nByte := nMotorByte_[nLptPort]
 DEFAULT nLptPort TO 1
 nByte := clearbit(nByte,nO1toO8)
 MfOutput(nLptPort,nByte)
RETURN
//
PROCEDURE MfMotorLeft(nLptPort,nMotorNr)
 MfMotor(nLptPort,nMotorNr,MCCW)
RETURN
//
PROCEDURE MfMotorRight(nLptPort,nMotorNr)
 MfMotor(nLptPort,nMotorNr,MCW)
RETURN
//
PROCEDURE MfMotorOff(nLptPort,nMotorNr)
 MfMotor(nLptPort,nMotorNr,MOFF)
RETURN
//
PROCEDURE MfMotorOn(nLptPort,nMotorNr)
 MfMotor(nLptPort,nMotorNr,MON)
RETURN
//
STATIC FUNCTION MfGetIOAddress(nLptPort)
 LOCAL xIOAddress:=NIL,csearch
 DEFAULT nLptPort TO 1
 IF file("mface.ini")
  readinival("mface.ini")
  csearch := "LPT" + str(nLptPort,1,0) + "_IO" // example: LPT3_IO = h3BC
  IF ini_exist(csearch)
   IF !empty(xIOAddress := ini_value(csearch))
    DO CASE
     CASE IS_NUM(xIOAddress)  // assume decimal number
     CASE IS_CHAR(xIOAddress)
      IF upper(left(xIOAddress,1)) == "H" // hex number follows "Hxxx"
       xIOAddress := padl(alltrim(substr(xIOAddress,2)),3,"0")
      ELSE
       IF !IS_HEX(xIOAddress) // als eerste letter geen ABCDEF
        xIOAddress := NIL
       ELSE
        xIOAddress := padl(alltrim(xIOAddress),3,"0")
       ENDIF
      ENDIF
     OTHERWISE
      xIOAddress := NIL
    ENDCASE
   ENDIF
  ENDIF
  clearinival()
 ENDIF
 IF empty(xIOAddress)
  DO CASE
   CASE nLptPort == 1
    xIOAddress := "378"
   CASE nLptPort == 2
    xIOAddress := "278"
   CASE nLptPort == 3
    xIOAddress := "3BC"
   CASE nLptPort == 4
    xIOAddress := "2BC"
   OTHERWISE
    xIOAddress := "378"
  ENDCASE
 ENDIF
RETURN xIOAddress
//
STATIC PROCEDURE MfShowUsedIF()
 LOCAL cmsg:="Multiface uses:",i
 FOR i := 1 TO 4
  IF lLptUsed_[i]
   cmsg += " LPT" + str(i,1,0) + " ,IO:" + cHexLptIO_[i] + "h"
  ENDIF
 NEXT
 rowmessage(cmsg,,,CR_MESSAGE,3)
RETURN
//
FUNCTION MfHexIO(nLptPort)
 LOCAL cHexIO:="---"
 DEFAULT nLptPort TO 1
 IF nLptPort > 0 .AND. lLptUsed_[nLptPort]
  cHexIO := cHexLptIO_[nLptPort]
 ENDIF
RETURN cHexIO
//
// einde mface.prg
//
